#include "CommonUtils/TabInterpolator.hpp"
#include "NuclearDataUtils/OtherUtils/Endf6FileStore.hpp"
#include "ContinuousEnergyXSUtils/AceDataParser/FastAceDataParser.hpp"

using namespace frendy;

////////////////////////////////////////////////////////////////////////////////////////////////////

void read_ace_file(string file_name, vector<int>& mt_list,
                   vector<vector<Integer> >& nbt_xs,   vector<vector<Integer> >& int_xs,
                   vector<vector<Real> >&    e_int_xs, vector<vector<Real> >&    xs_tab)
{
  //Read ACE file
  FastAceDataParser read_obj;
  read_obj.set_file_name(file_name);
  FastAceDataObject ace_obj = read_obj.get_ace_data_obj();

  int i_max = static_cast<int>(mt_list.size());
  if( i_max == 0 )
  {
    mt_list.push_back(  1); //total
    mt_list.push_back(  2); //scatter;
    mt_list.push_back(101); //disappearance

    vector<int> mt_list_add = ace_obj.get_react_type_list();
    for(int i=0; i<static_cast<int>(mt_list_add.size()); i++)
    {
      mt_list.push_back(mt_list_add[i]);
    }
    mt_list_add.clear();
    
    i_max   = static_cast<int>(mt_list.size());
  }

  //Resize xs data
  nbt_xs.resize(i_max);
  int_xs.resize(i_max);
  e_int_xs.resize(i_max);
  xs_tab.resize(i_max);
  
  for(int i=0; i<i_max; i++)
  {
    ace_obj.get_xs_data(mt_list[i], e_int_xs[i], xs_tab[i]);

    int j_max = static_cast<int>(e_int_xs[i].size());
    nbt_xs[i].resize(1);
    int_xs[i].resize(1);
    nbt_xs[i][0] = j_max;
    int_xs[i][0] = int_lin_lin;

    for(int j=0; j<j_max; j++)
    {
      e_int_xs[i][j] *= 1.0E6; //MeV -> eV
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void modify_xs_data(vector<vector<Real> >& e_ref,
                    vector<vector<Integer> >& nbt_xs,   vector<vector<Integer> >& int_xs,
                    vector<vector<Real> >&    e_int_xs, vector<vector<Real> >&    xs_tab)
{
  TabInterpolator ti_obj;

  int i_max = static_cast<int>(e_ref.size());
  for(int i=0; i<i_max; i++)
  {
    vector<Integer> nbt_mod, int_mod;
    vector<Real>    xs_mod;

    int j_max = static_cast<int>(e_ref[i].size());
    nbt_mod.push_back(j_max);
    int_mod.push_back(int_lin_lin);

    xs_mod.resize(j_max);
    for(int j=0; j<j_max; j++)
    {
      ti_obj.interpolation_tab1(e_ref[i][j], xs_mod[j], nbt_xs[i], int_xs[i], e_int_xs[i], xs_tab[i]);
    }
    nbt_xs[i]   = nbt_mod;
    int_xs[i]   = int_mod;
    e_int_xs[i] = e_ref[i];
    xs_tab[i]   = xs_mod;
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void output_result(string out_name, vector<Integer> xs_type_list, vector<vector<Real> >& ene_vec,
             vector<vector<Real> >& xs_ref_vec, vector<vector<Real> >& xs_comp_vec)
{
  int xs_data_no = static_cast<int>(xs_type_list.size());
  for(int i=0; i<xs_data_no; i++)
  {
    ostringstream  oss;
    oss << xs_type_list[i];

    string out_name_mod = out_name + "_mt";
    if( xs_type_list[i] < 10 )
    {
      out_name_mod = out_name_mod + "00";
    }
    else if( xs_type_list[i] < 100 )
    {
      out_name_mod = out_name_mod + "0";
    }
    out_name_mod = out_name_mod + oss.str() + ".out";

    ofstream fout;
    fout.open(out_name_mod.c_str());

    fout.precision(10);
    fout.setf(ios::showpoint);
    fout.setf(ios::scientific);

    int j_max = static_cast<int>(ene_vec[i].size());
    for(int j=0; j<j_max; j++)
    {
      Real xs_dif = xs_comp_vec[i][j] - xs_ref_vec[i][j];
      if( fabs(xs_ref_vec[i][j]) > min_value )
      {
        xs_dif /= xs_ref_vec[i][j];
      }

      fout << ene_vec[i][j] << "  "
           << xs_comp_vec[i][j] << "  " << xs_ref_vec[i][j] << "  " << xs_dif << endl;
    }
    fout.close();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
  if( argc <= 1 )
  {
    throw runtime_error("Please set reference ace file name.");
  }
  else if( argc <= 2 )
  {
    throw runtime_error("Please set comparing ace file name.");
  }
  else if( argc <= 3 )
  {
    throw runtime_error("Please set output file name.");
  }


  string  ace_file_name_ref  = argv[1];
  string  ace_file_name_comp = argv[2];
  string  output_name        = argv[3];

  vector<int> mt_list;
  mt_list.clear();
//  mt_list.push_back(  1); //total
//  mt_list.push_back(  2); //scatter;
//  mt_list.push_back(101); //disappearance


  vector<vector<Integer> > nbt_xs_r, nbt_xs_c, int_xs_r, int_xs_c;
  vector<vector<Real> >    e_int_xs_r, e_int_xs_c, xs_tab_r, xs_tab_c;

  cout << "  read_ace_file (ref)  " << ace_file_name_ref  << endl;
  read_ace_file(ace_file_name_ref,  mt_list,
                nbt_xs_r, int_xs_r, e_int_xs_r, xs_tab_r);

  cout << "  read_ace_file (comp) " << ace_file_name_comp << endl;
  read_ace_file(ace_file_name_comp,  mt_list,
                nbt_xs_c, int_xs_c, e_int_xs_c, xs_tab_c);

  cout << "  modify_xs_data" << endl;
  modify_xs_data(e_int_xs_r, nbt_xs_c, int_xs_c, e_int_xs_c, xs_tab_c);

  //output ene, xs_r, xs_c
  cout << "  output_results" << endl;
  output_result(output_name, mt_list, e_int_xs_r, xs_tab_r, xs_tab_c);
}

