#include "CommonUtils/TabInterpolator.hpp"
#include "CommonUtils/FileCommentRemover.hpp"
#include "EndfUtils/Endf6Parser/Endf6Parser.hpp"

using namespace frendy;

////////////////////////////////////////////////////////////////////////////////////////////////////

void read_input(string inp_name, string& out_name, string& ref_name, string& comp_name)
{
  FileCommentRemover rc_obj;
  string inp_name_mod = inp_name + "aaabbbcccdddeeefffggg.hhh";

  rc_obj.remove_comment(inp_name, inp_name_mod);

  ifstream fin;
  fin.open(inp_name_mod.c_str());
  fin >> out_name;
  fin >> ref_name;
  fin >> comp_name;
  cout << "Output file name            : " << out_name  << endl;
  cout << "Reference PENDF file name   : " << ref_name  << endl;
  cout << "Compare PENDF file name     : " << comp_name << endl;
  fin.close();
  remove(inp_name_mod.c_str());
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void read_mt_list( string file_name, vector<Integer>& mt_list )
{
  string mod_name = file_name + "aaabbbcccdddeeefff";

  FileCommentRemover remove_obj;
  remove_obj.remove_comment(file_name, mod_name);

  ifstream fin;
  fin.open(mod_name.c_str());

  int mt_val = 0;
  while( !fin.eof() )
  {
    fin >> mt_val;
    if( mt_val > 0 && mt_val < 1000 )
    {
      mt_list.push_back(mt_val);
      mt_val = 0;
    }
  }

  fin.close();
  remove(mod_name.c_str());
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void set_xs_type_list(string file_name,  vector<Integer>& xs_type_list)
{
  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_file_name_xs_only(file_name);

  //Get MF3 data
  vector<MF03Parser> mf03 = endf_obj.get_mf03_data();
  endf_obj.clear();

  xs_type_list.clear();
  for(int i=0; i<static_cast<int>(mf03.size()); i++)
  {
    Integer mt_no = mf03[i].get_mt_no();
    if( mt_no != unassigned_mt_no )
    {
      xs_type_list.push_back(mt_no);
    }
  }
}


////////////////////////////////////////////////////////////////////////////////////////////////////

void check_xs_type_list(vector<Integer>  xs_type_list, vector<Integer> xs_type_list_comp,
                        vector<Integer>& xs_type_list_mod)
{
  xs_type_list_mod.clear();

  int i_max = static_cast<int>(xs_type_list.size());
  int j_max = static_cast<int>(xs_type_list_comp.size());
  for(int i=0; i<i_max; i++)
  {
    Integer react_no = xs_type_list[i];

    int react_flg = 0;
    for(int j=0; j<j_max; j++)
    {
      if( xs_type_list_comp[j] == react_no )
      {
        react_flg = 1;
        xs_type_list_mod.push_back(react_no);
        break;
      }
    }

    if( react_flg == 0 )
    {
      ErrorManager err_obj;

      string class_name = "comp_xs_line.cpp";
      string func_name  = "check_xs_type_list";

      ostringstream oss;
      oss << react_no;
      string str_data = "Reaction type : " + oss.str();

      vector<string> err_com;
      err_com.push_back(str_data);
      err_com.push_back("This reaction type is not found in comparison results.");
      err_com.push_back("Comparison of this reaction type is skipped.");

      err_obj.output_caution(class_name, func_name, err_com);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void set_xs_data(string file_name,  vector<Integer> xs_type_list,
                 vector<vector<Integer> >& nbt_xs,   vector<vector<Integer> >& int_xs,
                 vector<vector<Real> >&    e_int_xs, vector<vector<Real> >&    xs_tab)
{
  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_file_name_xs_only(file_name);

  //Resize xs data
  int xs_data_no = static_cast<int>(xs_type_list.size());
  nbt_xs.resize(xs_data_no);
  int_xs.resize(xs_data_no);
  e_int_xs.resize(xs_data_no);
  xs_tab.resize(xs_data_no);

  //Get MF3 data
  vector<MF03Parser> mf03 = endf_obj.get_mf03_data();
  for(int i=0; i<static_cast<int>(mf03.size()); i++)
  {
    Integer mt_no = mf03[i].get_mt_no();
    for(int j=0; j<xs_data_no; j++)
    {
      if( mt_no == xs_type_list[j] )
      {
        nbt_xs[j]   = mf03[i].get_NBT_SIG();
        int_xs[j]   = mf03[i].get_INT_SIG();
        e_int_xs[j] = mf03[i].get_E_INT_SIG();
        xs_tab[j]   = mf03[i].get_SIG_TAB();
        break;
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void comp_xs(string ref_name, string comp_name, vector<Integer> xs_type_list, vector<vector<Real> >& ene_vec,
             vector<vector<Real> >& xs_ref_vec, vector<vector<Real> >& xs_comp_vec,
             vector<vector<Real> >& xs_dif_vec, vector<Real>& rms_vec)
{
  vector<vector<Integer> > nbt_xs_ref,   int_xs_ref, nbt_xs_comp, int_xs_comp;
  vector<vector<Real> >    e_int_xs_ref, xs_tab_ref, e_int_xs_comp, xs_tab_comp;
  set_xs_data(ref_name,  xs_type_list, nbt_xs_ref,  int_xs_ref,  e_int_xs_ref,  xs_tab_ref);
  set_xs_data(comp_name, xs_type_list, nbt_xs_comp, int_xs_comp, e_int_xs_comp, xs_tab_comp);

  ene_vec    = e_int_xs_ref;
  xs_ref_vec = xs_tab_ref;
  

  TabInterpolator ti_obj;
  int xs_data_no = static_cast<int>(xs_type_list.size());

  xs_comp_vec.clear();
  xs_dif_vec.clear();
  rms_vec.clear();
  xs_comp_vec.resize(xs_data_no);
  xs_dif_vec.resize(xs_data_no);
  rms_vec.resize(xs_data_no);
  for(int i=0; i<xs_data_no; i++)
  {
    rms_vec[i] = 0.0;

    int j_max = static_cast<int>(e_int_xs_ref[i].size());
    xs_comp_vec[i].resize(j_max);
    xs_dif_vec[i].resize(j_max);
    for(int j=0; j<j_max; j++)
    {
      xs_comp_vec[i][j] = 0.0;
      ti_obj.interpolation_tab1(e_int_xs_ref[i][j], xs_comp_vec[i][j],
                                nbt_xs_comp[i], int_xs_comp[i], e_int_xs_comp[i], xs_tab_comp[i]);

      if( fabs(xs_tab_ref[i][j]) > min_value )
      {
        xs_dif_vec[i][j] = (xs_comp_vec[i][j] - xs_tab_ref[i][j]) / xs_tab_ref[i][j];
      }
      else
      {
        xs_dif_vec[i][j] = (xs_comp_vec[i][j] - xs_tab_ref[i][j]);
      }

      rms_vec[i] += xs_dif_vec[i][j] * xs_dif_vec[i][j];
    }

    if( j_max > 0 )
    {
      rms_vec[i] = sqrt(rms_vec[i] / static_cast<Real>(j_max));
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void output_result(string out_name, vector<Integer> xs_type_list, vector<vector<Real> >& ene_vec,
             vector<vector<Real> >& xs_ref_vec, vector<vector<Real> >& xs_comp_vec,
             vector<vector<Real> >& xs_dif_vec)
{
  int xs_data_no = static_cast<int>(xs_type_list.size());
  for(int i=0; i<xs_data_no; i++)
  {
    ostringstream  oss;
    oss << xs_type_list[i];

    string out_name_mod = out_name + "_mt";
    if( xs_type_list[i] < 10 )
    {
      out_name_mod = out_name_mod + "00";
    }
    else if( xs_type_list[i] < 100 )
    {
      out_name_mod = out_name_mod + "0";
    }
    out_name_mod = out_name_mod + oss.str() + ".out";

    ofstream fout;
    fout.open(out_name_mod.c_str());

    fout.precision(10);
    fout.setf(ios::showpoint);
    fout.setf(ios::scientific);

    int j_max = static_cast<int>(ene_vec[i].size());
    for(int j=0; j<j_max; j++)
    {
      fout << ene_vec[i][j] << "  "
           << xs_comp_vec[i][j] << "  " << xs_ref_vec[i][j] << "  " << xs_dif_vec[i][j] << endl;
    }
    fout.close();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
  ErrorManager err_obj;
  //err_obj.set_err_mes_opt(err_obj.err_mes_debug);
  err_obj.set_err_mes_opt(err_obj.err_mes_default);
  //err_obj.set_err_mes_opt(err_obj.err_mes_skip_warning);

  if( argc <= 1 )
  {
    throw runtime_error("Please set input file name.");
  }

  string inp_name = argv[1];

  //Read input data
  string          out_name, ref_name, comp_name;
  read_input(inp_name, out_name, ref_name, comp_name);

  //Read MT number list
  vector<Integer> xs_type_list;
  xs_type_list.clear();
  if( argc > 2 )
  {
    string mt_list_name = argv[2];
    read_mt_list(mt_list_name, xs_type_list);

    if( static_cast<int>(xs_type_list.size()) == 0 )
    {
      cout << "Though user sets mt-list file, there is no mt data." << endl;
      cout << "This program compares in all mt data." << endl;
    }
  }

  if( static_cast<int>(xs_type_list.size()) == 0 )
  {
    //Read mt list from PENDF file
    vector<Integer> xs_type_list_comp, xs_type_list_mod;
    set_xs_type_list(ref_name,  xs_type_list);
    set_xs_type_list(comp_name, xs_type_list_comp);
    check_xs_type_list(xs_type_list, xs_type_list_comp, xs_type_list_mod);
    xs_type_list = xs_type_list_mod;
  }

  //Read cross section data and compare these data
  vector<vector<Real> > ene_vec, xs_ref_vec, xs_comp_vec, xs_dif_vec;
  vector<Real>          rms_vec;
  comp_xs(ref_name, comp_name, xs_type_list, ene_vec, xs_ref_vec, xs_comp_vec, xs_dif_vec, rms_vec);

  //Output comparison results
  output_result(out_name, xs_type_list, ene_vec, xs_ref_vec, xs_comp_vec, xs_dif_vec);


  int xs_data_no = static_cast<int>(xs_type_list.size());

  cout.precision(10);
  cout.setf(ios::scientific);
  cout << "Reaction type\tEnergy grid number\tRMS value [%]" << endl;
  for(int i=0; i<xs_data_no; i++)
  {
    cout << xs_type_list[i] << "\t" << ene_vec[i].size() << "\t" << rms_vec[i]*100.0 << endl;
  }

  return 0;
}

