#include "CommonUtils/TabInterpolator.hpp"
#include "NuclearDataUtils/OtherUtils/Endf6FileStore.hpp"
#include "ContinuousEnergyXSUtils/AceDataParser/FastAceDataParser.hpp"
#include "CommonUtils/FileCommentRemover.hpp"

using namespace frendy;

////////////////////////////////////////////////////////////////////////////////////////////////////

void read_mt_list( string file_name, vector<int>& mt_list )
{
  string mod_name = file_name + "aaabbbcccdddeeefff";

  FileCommentRemover remove_obj;
  remove_obj.remove_comment(file_name, mod_name);

  ifstream fin;
  fin.open(mod_name.c_str());

  int mt_val = 0;
  while( !fin.eof() )
  {
    fin >> mt_val;
    if( mt_val > 0 && mt_val < 1000 )
    {
      mt_list.push_back(mt_val);
      mt_val = 0;
    }
  }

  fin.close();
  remove(mod_name.c_str());
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void read_ace_file(string file_name, vector<int>& mt_list,
                   vector<vector<Integer> >& nbt_xs,   vector<vector<Integer> >& int_xs,
                   vector<vector<Real> >&    e_int_xs, vector<vector<Real> >&    xs_tab)
{
  //Read ACE file
  FastAceDataParser read_obj;
  read_obj.set_file_name(file_name);
  FastAceDataObject ace_obj = read_obj.get_ace_data_obj();

  int i_max = static_cast<int>(mt_list.size());
  if( i_max == 0 )
  {
    mt_list.push_back(  1); //total
    mt_list.push_back(  2); //scatter;
    mt_list.push_back(101); //disappearance

    vector<int> mt_list_add = ace_obj.get_react_type_list();
    for(int i=0; i<static_cast<int>(mt_list_add.size()); i++)
    {
      mt_list.push_back(mt_list_add[i]);
    }
    mt_list_add.clear();
    
    i_max   = static_cast<int>(mt_list.size());
  }

  //Resize xs data
  nbt_xs.resize(i_max);
  int_xs.resize(i_max);
  e_int_xs.resize(i_max);
  xs_tab.resize(i_max);
  
  for(int i=0; i<i_max; i++)
  {
    ace_obj.get_xs_data(mt_list[i], e_int_xs[i], xs_tab[i]);

    int j_max = static_cast<int>(e_int_xs[i].size());
    nbt_xs[i].resize(1);
    int_xs[i].resize(1);
    nbt_xs[i][0] = j_max;
    int_xs[i][0] = int_lin_lin;

    for(int j=0; j<j_max; j++)
    {
      e_int_xs[i][j] *= 1.0E6; //MeV -> eV
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void output_result(string out_name, vector<Integer> xs_type_list, 
                   vector<vector<Real> >& ene_vec, vector<vector<Real> >& xs_vec)
{
  int xs_data_no = static_cast<int>(xs_type_list.size());
  for(int i=0; i<xs_data_no; i++)
  {
    ostringstream  oss;
    oss << xs_type_list[i];

    string out_name_mod = out_name + "_mt";
    if( xs_type_list[i] < 10 )
    {
      out_name_mod = out_name_mod + "00";
    }
    else if( xs_type_list[i] < 100 )
    {
      out_name_mod = out_name_mod + "0";
    }
    out_name_mod = out_name_mod + oss.str() + ".xs";

    int j_max = static_cast<int>(ene_vec[i].size());
    if( j_max > 0 )
    {
      ofstream fout;
      fout.open(out_name_mod.c_str());

      fout.precision(10);
      fout.setf(ios::showpoint);
      fout.setf(ios::scientific);

      for(int j=0; j<j_max; j++)
      {
        fout << ene_vec[i][j] << "  " << xs_vec[i][j] << endl;
      }
      fout.close();
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
  if( argc <= 1 )
  {
    throw runtime_error("Please set ace file name.");
  }

  string  ace_file_name = argv[1];

  vector<int> mt_list;
  mt_list.clear();
//  mt_list.push_back(  1); //total
//  mt_list.push_back(  2); //scatter;
//  mt_list.push_back(101); //disappearance
  if( argc > 2 )
  {
    string mt_list_name = argv[2];
    read_mt_list(mt_list_name, mt_list);

    if( static_cast<int>(mt_list.size()) == 0 )
    {
      cout << "Though user sets mt-list file, there is no mt data." << endl;
      cout << "This program compares in all mt data." << endl;
    }
  }

  vector<vector<Integer> > nbt_xs,   int_xs;
  vector<vector<Real> >    e_int_xs, xs_tab;

  cout << "  read_ace_file " << ace_file_name  << endl;
  read_ace_file(ace_file_name,  mt_list,
                nbt_xs, int_xs, e_int_xs, xs_tab);

  //output ene, xs_r, xs_c
  cout << "  output_results" << endl;
  output_result(ace_file_name, mt_list, e_int_xs, xs_tab);
}

